@@ -1,7 +1,7 @@
 ﻿/*
  * Non-physical true random number generator based on timing jitter.
  *
- * Copyright Stephan Mueller <smueller@chronox.de>, 2014
+ * Copyright Stephan Mueller <smueller@chronox.de>, 2014 - 2019
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -58,10 +58,10 @@
 		      * require consumer to be updated (as long as this number
 		      * is zero, the API is not considered stable and can
 		      * change without a bump of the major version) */
-#define MINVERSION 0 /* API compatible, ABI may change, functional
+#define MINVERSION 1 /* API compatible, ABI may change, functional
 		      * enhancements only, consumer can be left unchanged if
 		      * enhancements are not considered */
-#define PATCHLEVEL 8 /* API / ABI compatible, no functional changes, no
+#define PATCHLEVEL 0 /* API / ABI compatible, no functional changes, no
 		      * enhancements, bug fixes only */
 
 static int Verbosity = 0;
@@ -146,7 +146,7 @@ static void parse_opts(int argc, char *argv[])
 			{"version", 0, 0, 0},
 			{0, 0, 0, 0}
 		};
-		c = getopt_long(argc, argv, "vph:", opts, &opt_index);
+		c = getopt_long(argc, argv, "vp:h", opts, &opt_index);
 		if (-1 == c)
 			break;
 		switch (c) {
@@ -266,9 +266,16 @@ static size_t write_random(struct kernel_rng *rng, char *buf, size_t len,
 
 static size_t gather_entropy(struct kernel_rng *rng)
 {
+	sigset_t blocking_set, previous_set;
 	char buf[(ENTROPYBYTES * OVERSAMPLINGFACTOR)];
 	size_t ret = 0;
 
+	sigemptyset(&previous_set);
+	sigemptyset(&blocking_set);
+	sigaddset(&blocking_set, SIGALRM);
+
+	sigprocmask(SIG_BLOCK, &blocking_set, &previous_set);
+
 	if (0 > jent_read_entropy(rng->ec, buf, sizeof(buf))) {
 		dolog(LOG_WARN, "Cannot read entropy");
 		return 0;
@@ -281,6 +288,8 @@ static size_t gather_entropy(struct kernel_rng *rng)
 		ret = sizeof(buf);
 	memset_secure(buf, 0, sizeof(buf));
 
+	sigprocmask(SIG_SETMASK, &previous_set, NULL);
+
 	return ret;
 }
 
@@ -359,6 +368,16 @@ static void sig_term(int sig)
 {
 	(void)sig;
 	dolog(LOG_DEBUG, "Shutting down cleanly\n");
+
+	/* Prevent the kernel from interfering with the shutdown */
+	signal(SIGALRM, SIG_IGN);
+
+	/* If we got another termination signal, just get killed */
+	signal(SIGHUP, SIG_DFL);
+	signal(SIGINT, SIG_DFL);
+	signal(SIGQUIT, SIG_DFL);
+	signal(SIGTERM, SIG_DFL);
+
 	dealloc();
 	exit(0);
 }
